@@ -167,6 +167,7 @@ static void bin_remove_messages (GstBin * bin, GstObject * src,
     GstMessageType types);
 static void gst_bin_recalc_func (GstBin * child, gpointer data);
 static gint bin_element_is_sink (GstElement * child, GstBin * bin);
+static gint bin_element_is_src (GstElement * child, GstBin * bin);
 
 static GstIterator *gst_bin_sort_iterator_new (GstBin * bin);
 
@@ -368,8 +369,10 @@ gst_bin_init (GstBin * bin)
   bin->clock_dirty = FALSE;
 
   /* Set up a bus for listening to child elements */
-  bus = g_object_new (gst_bus_get_type (), NULL);
+  bus = gst_bus_new ();
   bin->child_bus = bus;
+  GST_DEBUG_OBJECT (bin, "using bus %" GST_PTR_FORMAT " to listen to children",
+      bus);
   gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bin_bus_handler, bin);
 }
 
@@ -381,9 +384,10 @@ gst_bin_dispose (GObject * object)
   GST_CAT_DEBUG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
 
   bin_remove_messages (bin, NULL, GST_MESSAGE_ANY);
-  gst_object_unref (bin->child_bus);
-  bin->child_bus = NULL;
+
+  gst_object_replace ((GstObject **) & bin->child_bus, NULL);
   gst_object_replace ((GstObject **) & bin->provided_clock, NULL);
+  gst_object_replace ((GstObject **) & bin->clock_provider, NULL);
 
   while (bin->children) {
     gst_bin_remove (bin, GST_ELEMENT_CAST (bin->children->data));
@@ -510,7 +514,12 @@ gst_bin_provide_clock_func (GstElement * element)
   gst_object_replace ((GstObject **) & bin->clock_provider,
       (GstObject *) provider);
   bin->clock_dirty = FALSE;
-  GST_DEBUG_OBJECT (bin, "provided new clock %p", result);
+  GST_DEBUG_OBJECT (bin,
+      "provided new clock %" GST_PTR_FORMAT " by provider %" GST_PTR_FORMAT,
+      result, provider);
+  /* Provider is not being returned to caller, just the result */
+  if (provider)
+    gst_object_unref (provider);
   GST_OBJECT_UNLOCK (bin);
 
   gst_iterator_free (it);
@@ -620,7 +629,7 @@ bin_remove_messages (GstBin * bin, GstObject * src, GstMessageType types)
       gst_message_unref (message);
     } else {
       GST_DEBUG_OBJECT (GST_MESSAGE_SRC (message),
-          "not deleting message of types %d", types);
+          "not deleting message of type %d", GST_MESSAGE_TYPE (message));
     }
   }
 }
@@ -881,6 +890,7 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
       GST_OBJECT_FLAG_UNSET (bin, GST_ELEMENT_IS_SINK);
     }
   }
+
   /* if the clock provider for this element is removed, we lost
    * the clock as well, we need to inform the parent of this
    * so that it can select a new clock */
@@ -890,6 +900,7 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
     clock_message =
         gst_message_new_clock_lost (GST_OBJECT_CAST (bin), bin->provided_clock);
   }
+
   bin->state_dirty = TRUE;
   GST_OBJECT_UNLOCK (bin);
 
@@ -903,6 +914,9 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
 
   gst_element_set_bus (element, NULL);
 
+  /* Clear the clock we provided to the element */
+  gst_element_set_clock (element, NULL);
+
   /* we ref here because after the _unparent() the element can be disposed
    * and we still need it to reset the UNPARENTING flag and fire a signal. */
   gst_object_ref (element);
@@ -1141,6 +1155,71 @@ gst_bin_iterate_sinks (GstBin * bin)
   return result;
 }
 
+/* returns 0 when TRUE because this is a GCompareFunc */
+/* MT safe */
+static gint
+bin_element_is_src (GstElement * child, GstBin * bin)
+{
+  gboolean is_src = FALSE;
+
+  /* we lock the child here for the remainder of the function to
+   * get its name and flag safely. */
+  GST_OBJECT_LOCK (child);
+  if (!GST_OBJECT_FLAG_IS_SET (child, GST_ELEMENT_IS_SINK) &&
+      !child->numsinkpads) {
+    is_src = TRUE;
+  }
+
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin,
+      "child %s %s src", GST_OBJECT_NAME (child), is_src ? "is" : "is not");
+
+  GST_OBJECT_UNLOCK (child);
+  return is_src ? 0 : 1;
+}
+
+static gint
+src_iterator_filter (GstElement * child, GstBin * bin)
+{
+  if (bin_element_is_src (child, bin) == 0) {
+    /* returns 0 because this is a GCompareFunc */
+    return 0;
+  } else {
+    /* child carries a ref from gst_bin_iterate_elements -- drop if not passing
+       through */
+    gst_object_unref (child);
+    return 1;
+  }
+}
+
+/**
+ * gst_bin_iterate_sources:
+ * @bin: a #GstBin
+ *
+ * Gets an iterator for all elements in the bin that have the
+ * #GST_ELEMENT_IS_SRC flag set.
+ *
+ * Each element yielded by the iterator will have its refcount increased, so
+ * unref after use.
+ *
+ * MT safe.  Caller owns returned value.
+ *
+ * Returns: a #GstIterator of #GstElement, or NULL
+ */
+GstIterator *
+gst_bin_iterate_sources (GstBin * bin)
+{
+  GstIterator *children;
+  GstIterator *result;
+
+  g_return_val_if_fail (GST_IS_BIN (bin), NULL);
+
+  children = gst_bin_iterate_elements (bin);
+  result = gst_iterator_filter (children,
+      (GCompareFunc) src_iterator_filter, bin);
+
+  return result;
+}
+
 /*
  * MT safe
  */
@@ -1621,6 +1700,8 @@ gst_bin_element_set_state (GstBin * bin, GstElement * element, GstState pending)
 
   /* skip locked elements */
   if (G_UNLIKELY (locked)) {
+    GST_DEBUG_OBJECT (element,
+        "element is locked, pretending state change succeeded");
     ret = GST_STATE_CHANGE_SUCCESS;
     goto done;
   }
@@ -1784,8 +1865,15 @@ gst_bin_send_event (GstElement * element, GstEvent * event)
   gboolean res = TRUE;
   gboolean done = FALSE;
 
-  iter = gst_bin_iterate_sinks (bin);
-  GST_DEBUG_OBJECT (bin, "Sending event to sink children");
+  if (GST_EVENT_IS_DOWNSTREAM (event)) {
+    iter = gst_bin_iterate_sources (bin);
+    GST_DEBUG_OBJECT (bin, "Sending %s event to src children",
+        GST_EVENT_TYPE_NAME (event));
+  } else {
+    iter = gst_bin_iterate_sinks (bin);
+    GST_DEBUG_OBJECT (bin, "Sending %s event to sink children",
+        GST_EVENT_TYPE_NAME (event));
+  }
 
   while (!done) {
     gpointer data;
@@ -1793,22 +1881,24 @@ gst_bin_send_event (GstElement * element, GstEvent * event)
     switch (gst_iterator_next (iter, &data)) {
       case GST_ITERATOR_OK:
       {
-        GstElement *sink;
+        GstElement *child;
 
         gst_event_ref (event);
-        sink = GST_ELEMENT_CAST (data);
-        res &= gst_element_send_event (sink, event);
-        gst_object_unref (sink);
+        child = GST_ELEMENT_CAST (data);
+        res &= gst_element_send_event (child, event);
+        gst_object_unref (child);
         break;
       }
       case GST_ITERATOR_RESYNC:
         gst_iterator_resync (iter);
         res = TRUE;
         break;
-      default:
       case GST_ITERATOR_DONE:
         done = TRUE;
         break;
+      case GST_ITERATOR_ERROR:
+        g_assert_not_reached ();
+        break;
     }
   }
   gst_iterator_free (iter);
@@ -1997,6 +2087,13 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
       provided = (clock == bin->provided_clock);
       playing = (GST_STATE (bin) == GST_STATE_PLAYING);
       forward = playing & provided;
+      if (provided) {
+        GST_DEBUG_OBJECT (bin,
+            "Lost clock %" GST_PTR_FORMAT " provided by %" GST_PTR_FORMAT,
+            bin->provided_clock, bin->clock_provider);
+        gst_object_replace ((GstObject **) & bin->provided_clock, NULL);
+        gst_object_replace ((GstObject **) & bin->clock_provider, NULL);
+      }
       GST_DEBUG_OBJECT (bin, "provided %d, playing %d, forward %d",
           provided, playing, forward);
       GST_OBJECT_UNLOCK (bin);
@@ -2075,6 +2172,8 @@ bin_query_duration_fold (GstElement * item, GValue * ret, QueryFold * fold)
     if (duration > fold->max)
       fold->max = duration;
   }
+
+  gst_object_unref (item);
   return TRUE;
 }
 static void
@@ -2088,11 +2187,13 @@ bin_query_duration_done (GstBin * bin, QueryFold * fold)
 
   GST_DEBUG_OBJECT (bin, "max duration %" G_GINT64_FORMAT, fold->max);
 
+#if 0
   /* and cache now */
   GST_OBJECT_LOCK (bin);
   bin->messages = g_list_prepend (bin->messages,
       gst_message_new_duration (GST_OBJECT_CAST (bin), format, fold->max));
   GST_OBJECT_UNLOCK (bin);
+#endif
 }
 
 /* generic fold, return first valid result */
@@ -2106,6 +2207,8 @@ bin_query_generic_fold (GstElement * item, GValue * ret, QueryFold * fold)
     GST_DEBUG_OBJECT (item, "answered query %p", fold->query);
   }
 
+  gst_object_unref (item);
+
   /* and stop as soon as we have a valid result */
   return !res;
 }
