@@ -62,10 +62,11 @@
  * You can get and set a #GstClock on an element using gst_element_get_clock()
  * and gst_element_set_clock().
  * Some elements can provide a clock for the pipeline if
- * gst_element_provides_clock() returns TRUE. With the gst_element_provide_clock()
- * method one can retrieve the clock provided by such an element.
+ * gst_element_provides_clock() returns %TRUE. With the
+ * gst_element_provide_clock() method one can retrieve the clock provided by
+ * such an element.
  * Not all elements require a clock to operate correctly. If
- * gst_element_requires_clock() returns TRUE, a clock should be set on the
+ * gst_element_requires_clock() returns %TRUE, a clock should be set on the
  * element with gst_element_set_clock().
  *
  * Note that clock slection and distribution is normally handled by the
@@ -446,7 +447,7 @@ gst_element_get_clock (GstElement * element)
  * @element: a #GstElement.
  * @time: the base time to set.
  *
- * Set the base time of an element. See @gst_element_get_base_time().
+ * Set the base time of an element. See gst_element_get_base_time().
  *
  * MT safe.
  */
@@ -1197,21 +1198,26 @@ gst_element_send_event (GstElement * element, GstEvent * event)
   oclass = GST_ELEMENT_GET_CLASS (element);
 
   if (oclass->send_event) {
-    GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "send event on element %s",
-        GST_ELEMENT_NAME (element));
+    GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "send %s event on element %s",
+        GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
     result = oclass->send_event (element, event);
   } else {
-    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    GstPad *pad = GST_EVENT_IS_DOWNSTREAM (event) ?
+        gst_element_get_random_pad (element, GST_PAD_SRC) :
+        gst_element_get_random_pad (element, GST_PAD_SINK);
 
     if (pad) {
       GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS,
-          "pushing event to random pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+          "pushing %s event to random %s pad %s:%s",
+          GST_EVENT_TYPE_NAME (event),
+          (GST_PAD_DIRECTION (pad) == GST_PAD_SRC ? "src" : "sink"),
+          GST_DEBUG_PAD_NAME (pad));
 
       result = gst_pad_push_event (pad, event);
       gst_object_unref (pad);
     } else {
-      GST_CAT_INFO (GST_CAT_ELEMENT_PADS, "can't send event on element %s",
-          GST_ELEMENT_NAME (element));
+      GST_CAT_INFO (GST_CAT_ELEMENT_PADS, "can't send %s event on element %s",
+          GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
     }
   }
   return result;
@@ -1396,7 +1402,7 @@ no_bus:
  * _gst_element_error_printf:
  * @format: the printf-like format to use, or NULL
  *
- * This function is only used internally by the #gst_element_error macro.
+ * This function is only used internally by the gst_element_error() macro.
  *
  * Returns: a newly allocated string, or NULL if the format was NULL or ""
  *
@@ -1446,10 +1452,11 @@ void gst_element_message_full
   gchar *name;
   gchar *sent_text;
   gchar *sent_debug;
+  gboolean has_debug = TRUE;
   GstMessage *message = NULL;
 
   /* checks */
-  GST_DEBUG ("start");
+  GST_DEBUG_OBJECT (element, "start");
   g_return_if_fail (GST_IS_ELEMENT (element));
   g_return_if_fail ((type == GST_MESSAGE_ERROR) ||
       (type == GST_MESSAGE_WARNING));
@@ -1465,13 +1472,17 @@ void gst_element_message_full
   /* construct a sent_debug with extra information from source */
   if ((debug == NULL) || (debug[0] == 0)) {
     /* debug could have come from g_strdup_printf (""); */
-    sent_debug = NULL;
-  } else {
-    name = gst_object_get_path_string (GST_OBJECT (element));
-    sent_debug = g_strdup_printf ("%s(%d): %s: %s:\n%s",
-        file, line, function, name, debug ? debug : "");
-    g_free (name);
+    has_debug = FALSE;
   }
+
+  name = gst_object_get_path_string (GST_OBJECT (element));
+  if (has_debug)
+    sent_debug = g_strdup_printf ("%s(%d): %s (): %s:\n%s",
+        file, line, function, name, debug);
+  else
+    sent_debug = g_strdup_printf ("%s(%d): %s (): %s",
+        file, line, function, name);
+  g_free (name);
   g_free (debug);
 
   /* create gerror and post message */
@@ -1730,36 +1741,36 @@ interrupted:
  * @element: a #GstElement to get the state of.
  * @state: a pointer to #GstState to hold the state. Can be NULL.
  * @pending: a pointer to #GstState to hold the pending state.
- *           Can be NULL.
+ *           Can be %NULL.
  * @timeout: a #GstClockTime to specify the timeout for an async
- *           state change or GST_CLOCK_TIME_NONE for infinite timeout.
+ *           state change or %GST_CLOCK_TIME_NONE for infinite timeout.
  *
  * Gets the state of the element.
  *
  * For elements that performed an ASYNC state change, as reported by
- * #gst_element_set_state(), this function will block up to the
+ * gst_element_set_state(), this function will block up to the
  * specified timeout value for the state change to complete.
  * If the element completes the state change or goes into
  * an error, this function returns immediately with a return value of
- * #GST_STATE_CHANGE_SUCCESS or #GST_STATE_CHANGE_FAILURE respectively.
+ * %GST_STATE_CHANGE_SUCCESS or %GST_STATE_CHANGE_FAILURE respectively.
  *
- * For elements that did not return #GST_STATE_CHANGE_ASYNC, this function
+ * For elements that did not return %GST_STATE_CHANGE_ASYNC, this function
  * returns the current and pending state immediately.
  *
- * This function returns #GST_STATE_CHANGE_NO_PREROLL if the element
+ * This function returns %GST_STATE_CHANGE_NO_PREROLL if the element
  * successfully changed its state but is not able to provide data yet.
  * This mostly
  * happens for live sources that only produce data in the PLAYING state.
- * While the state change return is equivalent to #GST_STATE_CHANGE_SUCCESS, it
+ * While the state change return is equivalent to %GST_STATE_CHANGE_SUCCESS, it
  * is returned to the application to signal that some sink elements might not
  * be able to complete their state change because an element is not producing
  * data to complete the preroll. When setting the element to playing,
  * the preroll will complete and playback will start.
  *
- * Returns: #GST_STATE_CHANGE_SUCCESS if the element has no more pending state
- *          and the last state change succeeded, #GST_STATE_CHANGE_ASYNC if the
+ * Returns: %GST_STATE_CHANGE_SUCCESS if the element has no more pending state
+ *          and the last state change succeeded, %GST_STATE_CHANGE_ASYNC if the
  *          element is still performing a state change or
- *          #GST_STATE_CHANGE_FAILURE if the last state change failed.
+ *          %GST_STATE_CHANGE_FAILURE if the last state change failed.
  *
  * MT safe.
  */
@@ -1886,6 +1897,8 @@ gst_element_continue_state (GstElement * element, GstStateChangeReturn ret)
   transition = GST_STATE_TRANSITION (current, next);
 
   GST_STATE_NEXT (element) = next;
+  /* mark busy */
+  GST_STATE_RETURN (element) = GST_STATE_CHANGE_ASYNC;
   GST_OBJECT_UNLOCK (element);
 
   GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
@@ -1945,7 +1958,7 @@ complete:
  *
  * Brings the element to the lost state. The current state of the
  * element is copied to the pending state so that any call to
- * #gst_element_get_state() will return ASYNC.
+ * gst_element_get_state() will return ASYNC.
  *
  * This is mostly used for elements that lost their preroll buffer
  * in the PAUSED state after a flush, they become PAUSED again
@@ -2098,6 +2111,13 @@ gst_element_set_state_func (GstElement * element, GstState state)
   next = GST_STATE_GET_NEXT (current, state);
   /* now we store the next state */
   GST_STATE_NEXT (element) = next;
+  /* mark busy, we need to check that there is actually a state change
+   * to be done else we could accidentally override SUCCESS/NO_PREROLL and
+   * the default element change_state function has no way to know what the
+   * old value was... could consider this a FIXME...*/
+  if (current != next)
+    GST_STATE_RETURN (element) = GST_STATE_CHANGE_ASYNC;
+
   transition = GST_STATE_TRANSITION (current, next);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
@@ -2199,11 +2219,8 @@ gst_element_change_state (GstElement * element, GstStateChange transition)
   return ret;
 
 async:
-  GST_OBJECT_LOCK (element);
-  GST_STATE_RETURN (element) = ret;
   GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "exit async state change %d",
       ret);
-  GST_OBJECT_UNLOCK (element);
 
   return ret;
 
@@ -2279,7 +2296,10 @@ gst_element_pads_activate (GstElement * element, gboolean active)
   g_value_init (&ret, G_TYPE_BOOLEAN);
   g_value_set_boolean (&ret, TRUE);
 
-  iter = gst_element_iterate_src_pads (element);
+  if (active)
+    iter = gst_element_iterate_src_pads (element);
+  else
+    iter = gst_element_iterate_sink_pads (element);
   fold_ok = iterator_fold_with_resync
       (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
   gst_iterator_free (iter);
@@ -2288,7 +2308,10 @@ gst_element_pads_activate (GstElement * element, gboolean active)
     return FALSE;
   }
 
-  iter = gst_element_iterate_sink_pads (element);
+  if (active)
+    iter = gst_element_iterate_sink_pads (element);
+  else
+    iter = gst_element_iterate_src_pads (element);
   fold_ok = iterator_fold_with_resync
       (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
   gst_iterator_free (iter);
