@@ -17,6 +17,79 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gsterror
+ * @short_description: Categorized error messages
+ * @see_also: #GstMessage
+ *
+ * GStreamer elements can throw non-fatal warnings and fatal errors.
+ * Higher-level elements and applications can programatically filter
+ * the ones they are interested in or can recover from,
+ * and have a default handler handle the rest of them.
+ *
+ * The rest of this section will use the term <quote>error</quote>
+ * to mean both (non-fatal) warnings and (fatal) errors; they are treated
+ * similarly.
+ *
+ * Errors from elements are the combination of a #GError and a debug string.
+ * The #GError contains:
+ * - a domain type: CORE, LIBRARY, RESOURCE or STREAM
+ * - a code: an enum value specific to the domain
+ * - a translated, human-readable message
+ * - a non-translated additional debug string, which also contains
+ * - file and line information
+ *
+ * Elements do not have the context required to decide what to do with
+ * errors.  As such, they should only inform about errors, and stop their
+ * processing.  In short, an element doesn't know what it is being used for.
+ *
+ * It is the application or compound element using the given element that
+ * has more context about the use of the element.  The thrown errors should
+ * be inspected, and filtered if appropriate.
+ *
+ * An application is expected to, by default, present the user with a
+ * dialog box (or an equivalent) showing the error message.  The dialog
+ * should also allow a way to get at the additional debug information,
+ * so the user can provide bug reporting information.
+ *
+ * A compound element is expected to forward errors by default higher up
+ * the hierarchy; this is done by default in the same way as for other types
+ * of #GstMessage.
+ *
+ * When applications or compound elements trigger errors that they can
+ * recover from, they can filter out these errors and take appropriate action.
+ * For example, an application that gets an error from xvimagesink
+ * that indicates all XVideo ports are taken, the application can attempt
+ * to use another sink instead.
+ *
+ * Elements throw errors using the #GST_ELEMENT_ERROR convenience macro:
+ *
+ * <example>
+ * <title>Throwing an error</title>
+ *   <programlisting>
+ *     GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+ *       (_("No file name specified for reading.")), (NULL));
+ *   </programlisting>
+ * </example>
+ *
+ * Things to keep in mind:
+ * <itemizedlist>
+ *   <listitem><para>Don't go off inventing new error codes.  The ones
+ *     currently provided should be enough.  If you find your type of error
+ *     does not fit the current codes, you should use FAILED.</para></listitem>
+ *   <listitem><para>Don't provide a message if the default one suffices.
+ *     this keeps messages more uniform.  Use (NULL) - not forgetting the
+ *     parentheses.</para></listitem>
+ *   <listitem><para>If you do supply a custom message, it should be
+ *     marked for translation.  The message should start with a capital
+ *     and end with a period.</para></listitem>
+ *   <listitem><para>The debug string can be as you like.  Again, use (NULL)
+ *     if there's nothing to add - file and line number will still be
+ *     passed.  #GST_ERROR_SYSTEM can be used as a shortcut to give
+ *     debug information on a system call error.</para></listitem>
+ * </itemizedlist>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -26,10 +99,10 @@
 #include "gst-i18n-lib.h"
 
 #define TABLE(t, d, a, b) t[GST_ ## d ## _ERROR_ ## a] = g_strdup (b)
-#define QUARK_FUNC(string)						\
-GQuark gst_ ## string ## _error_quark (void) {				\
-  static GQuark quark;							\
-  if (!quark)								\
+#define QUARK_FUNC(string)                                              \
+GQuark gst_ ## string ## _error_quark (void) {                          \
+  static GQuark quark;                                                  \
+  if (!quark)                                                           \
     quark = g_quark_from_static_string ("gst-" # string "-error-quark"); \
   return quark; }
 
@@ -44,6 +117,9 @@ gst_g_error_get_type (void)
   return type;
 }
 
+#define FILE_A_BUG "  Please file a bug at "\
+    "http://bugzilla.gnome.org/enter_bug.cgi?product=GStreamer."
+
 /* initialize the dynamic table of translated core errors */
 static gchar **
 _gst_core_errors_init (void)
@@ -56,27 +132,27 @@ _gst_core_errors_init (void)
       N_("GStreamer encountered a general core library error."));
   TABLE (t, CORE, TOO_LAZY,
       N_("GStreamer developers were too lazy to assign an error code "
-          "to this error.  Please file a bug."));
+          "to this error." FILE_A_BUG));
   TABLE (t, CORE, NOT_IMPLEMENTED,
-      N_("Internal GStreamer error: code not implemented.  File a bug."));
+      N_("Internal GStreamer error: code not implemented." FILE_A_BUG));
   TABLE (t, CORE, STATE_CHANGE,
-      N_("Internal GStreamer error: state change failed.  File a bug."));
-  TABLE (t, CORE, PAD,
-      N_("Internal GStreamer error: pad problem.  File a bug."));
+      N_("Internal GStreamer error: state change failed." FILE_A_BUG));
+  TABLE (t, CORE, PAD, N_("Internal GStreamer error: pad problem." FILE_A_BUG));
   TABLE (t, CORE, THREAD,
-      N_("Internal GStreamer error: thread problem.  File a bug."));
-  TABLE (t, CORE, SCHEDULER,
-      N_("Internal GStreamer error: scheduler problem.  File a bug."));
+      N_("Internal GStreamer error: thread problem." FILE_A_BUG));
   TABLE (t, CORE, NEGOTIATION,
-      N_("Internal GStreamer error: negotiation problem.  File a bug."));
+      N_("Internal GStreamer error: negotiation problem." FILE_A_BUG));
   TABLE (t, CORE, EVENT,
-      N_("Internal GStreamer error: event problem.  File a bug."));
+      N_("Internal GStreamer error: event problem." FILE_A_BUG));
   TABLE (t, CORE, SEEK,
-      N_("Internal GStreamer error: seek problem.  File a bug."));
+      N_("Internal GStreamer error: seek problem." FILE_A_BUG));
   TABLE (t, CORE, CAPS,
-      N_("Internal GStreamer error: capabilities problem.  File a bug."));
-  TABLE (t, CORE, TAG,
-      N_("Internal GStreamer error: tag problem.  File a bug."));
+      N_("Internal GStreamer error: caps problem." FILE_A_BUG));
+  TABLE (t, CORE, TAG, N_("Internal GStreamer error: tag problem." FILE_A_BUG));
+  TABLE (t, CORE, MISSING_PLUGIN,
+      N_("Your GStreamer installation is missing a plug-in."));
+  TABLE (t, CORE, CLOCK,
+      N_("Internal GStreamer error: clock problem." FILE_A_BUG));
 
   return t;
 }
@@ -93,11 +169,10 @@ _gst_library_errors_init (void)
       N_("GStreamer encountered a general supporting library error."));
   TABLE (t, LIBRARY, TOO_LAZY,
       N_("GStreamer developers were too lazy to assign an error code "
-          "to this error.  Please file a bug."));
+          "to this error." FILE_A_BUG));
   TABLE (t, LIBRARY, INIT, N_("Could not initialize supporting library."));
   TABLE (t, LIBRARY, SHUTDOWN, N_("Could not close supporting library."));
-  TABLE (t, LIBRARY, SETTINGS, N_("Could not set settings."));
-  TABLE (t, LIBRARY, ENCODE, N_("Failed to encode image."));
+  TABLE (t, LIBRARY, SETTINGS, N_("Could not close supporting library."));
 
   return t;
 }
@@ -114,7 +189,7 @@ _gst_resource_errors_init (void)
       N_("GStreamer encountered a general resource error."));
   TABLE (t, RESOURCE, TOO_LAZY,
       N_("GStreamer developers were too lazy to assign an error code "
-          "to this error.  Please file a bug."));
+          "to this error." FILE_A_BUG));
   TABLE (t, RESOURCE, NOT_FOUND, N_("Resource not found."));
   TABLE (t, RESOURCE, BUSY, N_("Resource busy or not available."));
   TABLE (t, RESOURCE, OPEN_READ, N_("Could not open resource for reading."));
@@ -144,7 +219,7 @@ _gst_stream_errors_init (void)
       N_("GStreamer encountered a general stream error."));
   TABLE (t, STREAM, TOO_LAZY,
       N_("GStreamer developers were too lazy to assign an error code "
-          "to this error.  Please file a bug."));
+          "to this error." FILE_A_BUG));
   TABLE (t, STREAM, NOT_IMPLEMENTED,
       N_("Element doesn't implement handling of this stream. "
           "Please file a bug."));
@@ -157,25 +232,27 @@ _gst_stream_errors_init (void)
   TABLE (t, STREAM, ENCODE, N_("Could not encode stream."));
   TABLE (t, STREAM, DEMUX, N_("Could not demultiplex stream."));
   TABLE (t, STREAM, MUX, N_("Could not multiplex stream."));
-  TABLE (t, STREAM, FORMAT, N_("Stream is of the wrong format."));
 
   return t;
 }
 
-QUARK_FUNC (core)
-    QUARK_FUNC (library)
-    QUARK_FUNC (resource)
-    QUARK_FUNC (stream)
+QUARK_FUNC (core);
+QUARK_FUNC (library);
+QUARK_FUNC (resource);
+QUARK_FUNC (stream);
 
 /**
  * gst_error_get_message:
  * @domain: the GStreamer error domain this error belongs to.
  * @code: the error code belonging to the domain.
  *
+ * Get a string describing the error message in the current locale.
+ *
  * Returns: a newly allocated string describing the error message in the
  * current locale.
  */
-     gchar *gst_error_get_message (GQuark domain, gint code)
+gchar *
+gst_error_get_message (GQuark domain, gint code)
 {
   static gchar **gst_core_errors = NULL;
   static gchar **gst_library_errors = NULL;
