@@ -667,7 +667,6 @@ gst_index_add_entry (GstIndex * index, GstIndexEntry * entry)
  * @flags: optinal flags for this entry
  * @n: number of associations
  * @list: list of associations
- * @...: other format/value pairs or 0 to end the list
  *
  * Associate given format/value pairs with each other.
  *
@@ -675,7 +674,7 @@ gst_index_add_entry (GstIndex * index, GstIndexEntry * entry)
  */
 GstIndexEntry *
 gst_index_add_associationv (GstIndex * index, gint id, GstAssocFlags flags,
-    int n, const GstIndexAssociation * list)
+    gint n, const GstIndexAssociation * list)
 {
   GstIndexEntry *entry;
 
