@@ -20,6 +20,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstxml
+ * @short_description: XML save/restore operations of pipelines
+ *
+ * GStreamer pipelines can be saved to xml files using gst_xml_write_file().
+ * They can be loaded back using gst_xml_parse_doc() / gst_xml_parse_file() / 
+ * gst_xml_parse_memory().
+ * Additionally one can load saved pipelines into the gst-editor to inspect the
+ * graph.
+ *
+ * #GstElement implementations need to override gst_object_save_thyself() and
+ * gst_object_restore_thyself().
+ */
+
 #include "gst_private.h"
 
 #include "gstxml.h"
@@ -75,7 +89,17 @@ gst_xml_class_init (GstXMLClass * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_OBJECT);
 
-  /* FIXME G_TYPE_POINTER should be GType of xmlNodePtr */
+  /* FIXME G_TYPE_POINTER should be GType of xmlNodePtr
+   * (ensonic) can't be fixed, as libxml does not use GObject (unfortunately)
+   */
+  /**
+   * GstXML::object-loaded:
+   * @xml: the xml persistence instance
+   * @object: the object that has been loaded
+   * @xml_node: the related xml_node pointer to the document tree
+   *
+   * Signals that a new object has been deserialized.
+   */
   gst_xml_signals[OBJECT_LOADED] =
       g_signal_new ("object-loaded", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstXMLClass, object_loaded), NULL,
@@ -118,16 +142,16 @@ gst_xml_write (GstElement * element)
   xmlNodePtr elementnode;
   xmlNsPtr gst_ns;
 
-  doc = xmlNewDoc ((guchar *) "1.0");
+  doc = xmlNewDoc ((xmlChar *) "1.0");
 
-  doc->xmlRootNode = xmlNewDocNode (doc, NULL, (guchar *) "gstreamer", NULL);
+  doc->xmlRootNode = xmlNewDocNode (doc, NULL, (xmlChar *) "gstreamer", NULL);
 
   gst_ns =
       xmlNewNs (doc->xmlRootNode,
-      (guchar *) "http://gstreamer.net/gst-core/1.0/", (guchar *) "gst");
+      (xmlChar *) "http://gstreamer.net/gst-core/1.0/", (xmlChar *) "gst");
 
-  elementnode =
-      xmlNewChild (doc->xmlRootNode, gst_ns, (guchar *) "element", NULL);
+  elementnode = xmlNewChild (doc->xmlRootNode, gst_ns, (xmlChar *) "element",
+      NULL);
 
   gst_object_save_thyself (GST_OBJECT (element), elementnode);
 
@@ -221,7 +245,7 @@ gst_xml_parse_doc (GstXML * xml, xmlDocPtr doc, const guchar * root)
     return FALSE;
   }
   ns = xmlSearchNsByHref (doc, cur,
-      (guchar *) "http://gstreamer.net/gst-core/1.0/");
+      (xmlChar *) "http://gstreamer.net/gst-core/1.0/");
   if (ns == NULL) {
     g_warning ("gstxml: document of wrong type, core namespace not found\n");
     return FALSE;
@@ -254,6 +278,7 @@ gst_xml_parse_doc (GstXML * xml, xmlDocPtr doc, const guchar * root)
   return TRUE;
 }
 
+/* FIXME 0.9: Why guchar*? */
 /**
  * gst_xml_parse_file:
  * @xml: a pointer to a GstXML object
@@ -288,6 +313,7 @@ gst_xml_parse_file (GstXML * xml, const guchar * fname, const guchar * root)
   return gst_xml_parse_doc (xml, doc, root);
 }
 
+/* FIXME 0.9: guchar* */
 /**
  * gst_xml_parse_memory:
  * @xml: a pointer to a GstXML object
@@ -310,7 +336,7 @@ gst_xml_parse_memory (GstXML * xml, guchar * buffer, guint size,
 
   doc = xmlParseMemory ((char *) buffer, size);
 
-  return gst_xml_parse_doc (xml, doc, (const guchar *) root);
+  return gst_xml_parse_doc (xml, doc, (const xmlChar *) root);
 }
 
 static void
@@ -340,6 +366,7 @@ gst_xml_get_topelements (GstXML * xml)
   return xml->topelements;
 }
 
+/* FIXME 0.9: why is the arg guchar* instead of gchar*? */
 /**
  * gst_xml_get_element:
  * @xml: The GstXML to get the element from
@@ -368,11 +395,11 @@ gst_xml_get_element (GstXML * xml, const guchar * name)
     GstElement *top = GST_ELEMENT (topelements->data);
 
     GST_DEBUG ("gstxml: getting element \"%s\"", name);
-    if (!strcmp ((char *) GST_ELEMENT_NAME (top), (char *) name)) {
+    if (!strcmp (GST_ELEMENT_NAME (top), (char *) name)) {
       return top;
     } else {
       if (GST_IS_BIN (top)) {
-        element = gst_bin_get_by_name (GST_BIN (top), (char *) name);
+        element = gst_bin_get_by_name (GST_BIN (top), (gchar *) name);
 
         if (element)
           return element;
@@ -397,15 +424,15 @@ gst_xml_make_element (xmlNodePtr cur, GstObject * parent)
 {
   xmlNodePtr children = cur->xmlChildrenNode;
   GstElement *element;
-  guchar *name = NULL;
-  guchar *type = NULL;
+  gchar *name = NULL;
+  gchar *type = NULL;
 
   /* first get the needed tags to construct the element */
   while (children) {
     if (!strcmp ((char *) children->name, "name")) {
-      name = xmlNodeGetContent (children);
+      name = (gchar *) xmlNodeGetContent (children);
     } else if (!strcmp ((char *) children->name, "type")) {
-      type = xmlNodeGetContent (children);
+      type = (gchar *) xmlNodeGetContent (children);
     }
     children = children->next;
   }
@@ -414,7 +441,7 @@ gst_xml_make_element (xmlNodePtr cur, GstObject * parent)
 
   GST_CAT_INFO (GST_CAT_XML, "loading \"%s\" of type \"%s\"", name, type);
 
-  element = gst_element_factory_make ((char *) type, (char *) name);
+  element = gst_element_factory_make (type, name);
 
   g_return_val_if_fail (element != NULL, NULL);
 
