@@ -51,7 +51,7 @@ gst_dp_crc (const guint8 * buffer, register guint length)
 {
   static gboolean initialized = FALSE;
   static guint16 crc_table[256];
-  guint16 crc_register;
+  guint16 crc_register = CRC_INIT;
   unsigned long i, j, k;
 
   if (!initialized) {
@@ -66,8 +66,6 @@ gst_dp_crc (const guint8 * buffer, register guint length)
     initialized = TRUE;
   }
 
-  crc_register = CRC_INIT;      /* always init register */
-
   /* calc CRC */
   for (; length--;) {
     crc_register = (guint16) ((crc_register << 8) ^
@@ -83,7 +81,7 @@ gst_dp_dump_byte_array (guint8 * array, guint length)
 {
   int i;
   int n = 8;                    /* number of bytes per line */
-  gchar *line = g_malloc (3 * n);
+  gchar *line = g_malloc0 (3 * n + 1);
 
   GST_LOG ("dumping byte array of length %d", length);
   for (i = 0; i < length; ++i) {
@@ -103,8 +101,8 @@ gst_dp_dump_byte_array (guint8 * array, guint length)
  *
  * Initialize GStreamer Data Protocol library.
  *
- * Should be called before using these functions; either from source linking
- * to this source file or from plugin_init.
+ * Should be called before using these functions from source linking
+ * to this source file.
  */
 void
 gst_dp_init (void)
@@ -126,6 +124,8 @@ gst_dp_init (void)
  * gst_dp_header_payload_length:
  * @header: the byte header of the packet array
  *
+ * Get the length of the payload described by @header.
+ *
  * Returns: the length of the payload this header describes.
  */
 guint32
@@ -138,6 +138,8 @@ gst_dp_header_payload_length (const guint8 * header)
  * gst_dp_header_payload_type:
  * @header: the byte header of the packet array
  *
+ * Get the type of the payload described by @header.
+ *
  * Returns: the #GstDPPayloadType the payload this header describes.
  */
 GstDPPayloadType
@@ -167,35 +169,33 @@ gst_dp_header_from_buffer (const GstBuffer * buffer, GstDPHeaderFlag flags,
   guint16 flags_mask;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (GST_BUFFER_REFCOUNT_VALUE (buffer) > 0, FALSE);
   g_return_val_if_fail (header, FALSE);
 
   *length = GST_DP_HEADER_LENGTH;
-  h = g_malloc (GST_DP_HEADER_LENGTH);
+  h = g_malloc0 (GST_DP_HEADER_LENGTH);
 
   /* version, flags, type */
   h[0] = (guint8) GST_DP_VERSION_MAJOR;
   h[1] = (guint8) GST_DP_VERSION_MINOR;
   h[2] = (guint8) flags;
-  h[3] = GST_DP_PAYLOAD_BUFFER;
+  h[3] = 0;                     /* padding byte */
+  GST_WRITE_UINT16_BE (h + 4, GST_DP_PAYLOAD_BUFFER);
 
   /* buffer properties */
-  GST_WRITE_UINT32_BE (h + 4, GST_BUFFER_SIZE (buffer));
-  GST_WRITE_UINT64_BE (h + 8, GST_BUFFER_TIMESTAMP (buffer));
-  GST_WRITE_UINT64_BE (h + 16, GST_BUFFER_DURATION (buffer));
-  GST_WRITE_UINT64_BE (h + 24, GST_BUFFER_OFFSET (buffer));
-  GST_WRITE_UINT64_BE (h + 32, GST_BUFFER_OFFSET_END (buffer));
+  GST_WRITE_UINT32_BE (h + 6, GST_BUFFER_SIZE (buffer));
+  GST_WRITE_UINT64_BE (h + 10, GST_BUFFER_TIMESTAMP (buffer));
+  GST_WRITE_UINT64_BE (h + 18, GST_BUFFER_DURATION (buffer));
+  GST_WRITE_UINT64_BE (h + 26, GST_BUFFER_OFFSET (buffer));
+  GST_WRITE_UINT64_BE (h + 34, GST_BUFFER_OFFSET_END (buffer));
 
   /* data flags */
   /* we only copy KEY_UNIT,DELTA_UNIT and IN_CAPS flags */
-  flags_mask = GST_DATA_FLAG_SHIFT (GST_BUFFER_KEY_UNIT) |
-      GST_DATA_FLAG_SHIFT (GST_BUFFER_IN_CAPS) |
-      GST_DATA_FLAG_SHIFT (GST_BUFFER_DELTA_UNIT);
+  flags_mask = GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_IN_CAPS |
+      GST_BUFFER_FLAG_DELTA_UNIT;
 
-  GST_WRITE_UINT16_BE (h + 40, GST_BUFFER_FLAGS (buffer) & flags_mask);
+  GST_WRITE_UINT16_BE (h + 42, GST_BUFFER_FLAGS (buffer) & flags_mask);
 
   /* ABI padding */
-  GST_WRITE_UINT16_BE (h + 42, (guint64) 0);
   GST_WRITE_UINT32_BE (h + 44, (guint64) 0);
   GST_WRITE_UINT64_BE (h + 48, (guint64) 0);
 
@@ -203,15 +203,15 @@ gst_dp_header_from_buffer (const GstBuffer * buffer, GstDPHeaderFlag flags,
   crc = 0;
   if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
     /* we don't crc the last four bytes of the header since they are crc's */
-    crc = gst_dp_crc (h, 56);
+    crc = gst_dp_crc (h, 58);
   }
-  GST_WRITE_UINT16_BE (h + 56, crc);
+  GST_WRITE_UINT16_BE (h + 58, crc);
 
   crc = 0;
   if (flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
     crc = gst_dp_crc (GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
   }
-  GST_WRITE_UINT16_BE (h + 58, crc);
+  GST_WRITE_UINT16_BE (h + 60, crc);
 
   GST_LOG ("created header from buffer:");
   gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
@@ -237,7 +237,7 @@ gst_dp_packet_from_caps (const GstCaps * caps, GstDPHeaderFlag flags,
 {
   guint8 *h;
   guint16 crc;
-  gchar *string;
+  guchar *string;
 
   /* FIXME: GST_IS_CAPS doesn't work
      g_return_val_if_fail (GST_IS_CAPS (caps), FALSE); */
@@ -246,39 +246,40 @@ gst_dp_packet_from_caps (const GstCaps * caps, GstDPHeaderFlag flags,
   g_return_val_if_fail (payload, FALSE);
 
   *length = GST_DP_HEADER_LENGTH;
-  h = g_malloc (GST_DP_HEADER_LENGTH);
+  h = g_malloc0 (GST_DP_HEADER_LENGTH);
 
-  string = gst_caps_to_string (caps);
+  string = (guchar *) gst_caps_to_string (caps);
 
   /* version, flags, type */
   h[0] = (guint8) GST_DP_VERSION_MAJOR;
   h[1] = (guint8) GST_DP_VERSION_MINOR;
   h[2] = (guint8) flags;
-  h[3] = GST_DP_PAYLOAD_CAPS;
+  h[3] = 0;                     /* padding bytes */
+  GST_WRITE_UINT16_BE (h + 4, GST_DP_PAYLOAD_CAPS);
 
   /* buffer properties */
-  GST_WRITE_UINT32_BE (h + 4, strlen (string) + 1);     /* include trailing 0 */
-  GST_WRITE_UINT64_BE (h + 8, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 16, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 24, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 32, (guint64) 0);
+  GST_WRITE_UINT32_BE (h + 6, strlen ((gchar *) string) + 1);   /* include trailing 0 */
+  GST_WRITE_UINT64_BE (h + 10, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 18, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 26, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 34, (guint64) 0);
 
   /* ABI padding */
-  GST_WRITE_UINT64_BE (h + 40, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 48, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 42, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 50, (guint64) 0);
 
   /* CRC */
   crc = 0;
   if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
-    crc = gst_dp_crc (h, 56);
+    crc = gst_dp_crc (h, 58);
   }
-  GST_WRITE_UINT16_BE (h + 56, crc);
+  GST_WRITE_UINT16_BE (h + 58, crc);
 
   crc = 0;
   if (flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
-    crc = gst_dp_crc (string, strlen (string) + 1);
+    crc = gst_dp_crc (string, strlen ((gchar *) string) + 1);
   }
-  GST_WRITE_UINT16_BE (h + 58, crc);
+  GST_WRITE_UINT16_BE (h + 60, crc);
 
   GST_LOG ("created header from caps:");
   gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
@@ -323,40 +324,35 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
       g_free (h);
       return FALSE;
     case GST_EVENT_EOS:
-    case GST_EVENT_FLUSH:
-    case GST_EVENT_EMPTY:
-    case GST_EVENT_DISCONTINUOUS:
-      GST_WRITE_UINT64_BE (h + 8, GST_EVENT_TIMESTAMP (event));
+    case GST_EVENT_FLUSH_START:
+    case GST_EVENT_FLUSH_STOP:
+    case GST_EVENT_NEWSEGMENT:
       pl_length = 0;
       *payload = NULL;
       break;
     case GST_EVENT_SEEK:
-      pl_length = 4 + 8 + 4;
+    {
+      gdouble rate;
+      GstFormat format;
+      GstSeekFlags flags;
+      GstSeekType cur_type, stop_type;
+      gint64 cur, stop;
+
+      gst_event_parse_seek ((GstEvent *) event, &rate, &format, &flags,
+          &cur_type, &cur, &stop_type, &stop);
+
+      pl_length = 4 + 4 + 4 + 8 + 4 + 8;
       *payload = g_malloc0 (pl_length);
-      GST_WRITE_UINT32_BE (*payload, (guint32) GST_EVENT_SEEK_TYPE (event));
-      GST_WRITE_UINT64_BE (*payload + 4,
-          (guint64) GST_EVENT_SEEK_OFFSET (event));
-      GST_WRITE_UINT32_BE (*payload + 12,
-          (guint32) GST_EVENT_SEEK_ACCURACY (event));
-      break;
-    case GST_EVENT_SEEK_SEGMENT:
-      pl_length = 4 + 8 + 8 + 4;
-      *payload = g_malloc0 (pl_length);
-      GST_WRITE_UINT32_BE (*payload, (guint32) GST_EVENT_SEEK_TYPE (event));
-      GST_WRITE_UINT64_BE (*payload + 4,
-          (guint64) GST_EVENT_SEEK_OFFSET (event));
-      GST_WRITE_UINT64_BE (*payload + 12,
-          (guint64) GST_EVENT_SEEK_ENDOFFSET (event));
-      GST_WRITE_UINT32_BE (*payload + 20,
-          (guint32) GST_EVENT_SEEK_ACCURACY (event));
+      /* FIXME write rate */
+      GST_WRITE_UINT32_BE (*payload, (guint32) format);
+      GST_WRITE_UINT32_BE (*payload + 4, (guint32) flags);
+      GST_WRITE_UINT32_BE (*payload + 8, (guint32) cur_type);
+      GST_WRITE_UINT64_BE (*payload + 12, (guint64) cur);
+      GST_WRITE_UINT32_BE (*payload + 20, (guint32) stop_type);
+      GST_WRITE_UINT64_BE (*payload + 24, (guint64) stop);
       break;
+    }
     case GST_EVENT_QOS:
-    case GST_EVENT_SEGMENT_DONE:
-    case GST_EVENT_SIZE:
-    case GST_EVENT_RATE:
-    case GST_EVENT_FILLER:
-    case GST_EVENT_TS_OFFSET:
-    case GST_EVENT_INTERRUPT:
     case GST_EVENT_NAVIGATION:
     case GST_EVENT_TAG:
       g_warning ("Unhandled event type %d, ignoring", GST_EVENT_TYPE (event));
@@ -372,30 +368,32 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
   h[0] = (guint8) GST_DP_VERSION_MAJOR;
   h[1] = (guint8) GST_DP_VERSION_MINOR;
   h[2] = (guint8) flags;
-  h[3] = GST_DP_PAYLOAD_EVENT_NONE + GST_EVENT_TYPE (event);
+  h[3] = 0;                     /* padding byte */
+  GST_WRITE_UINT16_BE (h + 4,
+      GST_DP_PAYLOAD_EVENT_NONE + GST_EVENT_TYPE (event));
 
   /* length */
-  GST_WRITE_UINT32_BE (h + 4, (guint32) pl_length);
+  GST_WRITE_UINT32_BE (h + 6, (guint32) pl_length);
   /* timestamp */
-  GST_WRITE_UINT64_BE (h + 8, GST_EVENT_TIMESTAMP (event));
+  GST_WRITE_UINT64_BE (h + 10, GST_EVENT_TIMESTAMP (event));
 
   /* ABI padding */
-  GST_WRITE_UINT64_BE (h + 40, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 48, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 42, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 50, (guint64) 0);
 
   /* CRC */
   crc = 0;
   if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
-    crc = gst_dp_crc (h, 56);
+    crc = gst_dp_crc (h, 58);
   }
-  GST_WRITE_UINT16_BE (h + 56, crc);
+  GST_WRITE_UINT16_BE (h + 58, crc);
 
   crc = 0;
   /* events can have a NULL payload */
   if (*payload && flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
-    crc = gst_dp_crc (*payload, strlen (*payload) + 1);
+    crc = gst_dp_crc (*payload, strlen ((gchar *) * payload) + 1);
   }
-  GST_WRITE_UINT16_BE (h + 58, crc);
+  GST_WRITE_UINT16_BE (h + 60, crc);
 
   GST_LOG ("created header from event:");
   gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
@@ -423,7 +421,7 @@ gst_dp_buffer_from_header (guint header_length, const guint8 * header)
   GstBuffer *buffer;
 
   g_return_val_if_fail (GST_DP_HEADER_PAYLOAD_TYPE (header) ==
-      GST_DP_PAYLOAD_BUFFER, FALSE);
+      GST_DP_PAYLOAD_BUFFER, NULL);
   buffer =
       gst_buffer_new_and_alloc ((guint) GST_DP_HEADER_PAYLOAD_LENGTH (header));
   GST_BUFFER_TIMESTAMP (buffer) = GST_DP_HEADER_TIMESTAMP (header);
@@ -450,15 +448,16 @@ gst_dp_caps_from_packet (guint header_length, const guint8 * header,
     const guint8 * payload)
 {
   GstCaps *caps;
-  const gchar *string;
+  gchar *string;
 
-  g_return_val_if_fail (header, FALSE);
-  g_return_val_if_fail (payload, FALSE);
+  g_return_val_if_fail (header, NULL);
+  g_return_val_if_fail (payload, NULL);
   g_return_val_if_fail (GST_DP_HEADER_PAYLOAD_TYPE (header) ==
-      GST_DP_PAYLOAD_CAPS, FALSE);
+      GST_DP_PAYLOAD_CAPS, NULL);
 
-  string = payload;
+  string = g_strndup ((gchar *) payload, GST_DP_HEADER_PAYLOAD_LENGTH (header));
   caps = gst_caps_from_string (string);
+  g_free (string);
   return caps;
 }
 
@@ -479,7 +478,7 @@ gst_dp_event_from_packet (guint header_length, const guint8 * header,
   GstEvent *event = NULL;
   GstEventType type;
 
-  g_return_val_if_fail (header, FALSE);
+  g_return_val_if_fail (header, NULL);
   /* payload can be NULL, e.g. for an EOS event */
 
   type = GST_DP_HEADER_PAYLOAD_TYPE (header) - GST_DP_PAYLOAD_EVENT_NONE;
@@ -488,48 +487,35 @@ gst_dp_event_from_packet (guint header_length, const guint8 * header,
       g_warning ("Unknown event, ignoring");
       return FALSE;
     case GST_EVENT_EOS:
-    case GST_EVENT_FLUSH:
-    case GST_EVENT_EMPTY:
-    case GST_EVENT_DISCONTINUOUS:
-      event = gst_event_new (type);
+    case GST_EVENT_FLUSH_START:
+    case GST_EVENT_FLUSH_STOP:
+    case GST_EVENT_NEWSEGMENT:
+      event = gst_event_new_custom (type, NULL);
       GST_EVENT_TIMESTAMP (event) = GST_DP_HEADER_TIMESTAMP (header);
       break;
     case GST_EVENT_SEEK:
     {
-      GstSeekType type;
-      gint64 offset;
-      GstSeekAccuracy accuracy;
+      gdouble rate;
+      GstFormat format;
+      GstSeekFlags flags;
+      GstSeekType cur_type, stop_type;
+      gint64 cur, stop;
 
-      type = (GstSeekType) GST_READ_UINT32_BE (payload);
-      offset = (gint64) GST_READ_UINT64_BE (payload + 4);
-      accuracy = (GstSeekAccuracy) GST_READ_UINT32_BE (payload + 12);
-      event = gst_event_new_seek (type, offset);
-      GST_EVENT_TIMESTAMP (event) = GST_DP_HEADER_TIMESTAMP (header);
-      GST_EVENT_SEEK_ACCURACY (event) = accuracy;
-      break;
-    }
-    case GST_EVENT_SEEK_SEGMENT:
-    {
-      GstSeekType type;
-      gint64 offset, endoffset;
-      GstSeekAccuracy accuracy;
+      /* FIXME, read rate */
+      rate = 1.0;
+      format = (GstFormat) GST_READ_UINT32_BE (payload);
+      flags = (GstSeekFlags) GST_READ_UINT32_BE (payload + 4);
+      cur_type = (GstSeekType) GST_READ_UINT32_BE (payload + 8);
+      cur = (gint64) GST_READ_UINT64_BE (payload + 12);
+      stop_type = (GstSeekType) GST_READ_UINT32_BE (payload + 20);
+      stop = (gint64) GST_READ_UINT64_BE (payload + 24);
 
-      type = (GstSeekType) GST_READ_UINT32_BE (payload);
-      offset = (gint64) GST_READ_UINT64_BE (payload + 4);
-      endoffset = (gint64) GST_READ_UINT64_BE (payload + 12);
-      accuracy = (GstSeekAccuracy) GST_READ_UINT32_BE (payload + 20);
-      event = gst_event_new_segment_seek (type, offset, endoffset);
+      event = gst_event_new_seek (rate, format, flags, cur_type, cur,
+          stop_type, stop);
       GST_EVENT_TIMESTAMP (event) = GST_DP_HEADER_TIMESTAMP (header);
-      GST_EVENT_SEEK_ACCURACY (event) = accuracy;
       break;
     }
     case GST_EVENT_QOS:
-    case GST_EVENT_SEGMENT_DONE:
-    case GST_EVENT_SIZE:
-    case GST_EVENT_RATE:
-    case GST_EVENT_FILLER:
-    case GST_EVENT_TS_OFFSET:
-    case GST_EVENT_INTERRUPT:
     case GST_EVENT_NAVIGATION:
     case GST_EVENT_TAG:
       g_warning ("Unhandled event type %d, ignoring", GST_EVENT_TYPE (event));
@@ -621,18 +607,3 @@ gst_dp_validate_packet (guint header_length, const guint8 * header,
 
   return TRUE;
 }
-
-/*** PLUGIN STUFF ***/
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  gst_dp_init ();
-
-  return TRUE;
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "gstdataprotocol",
-    "a data protocol to serialize buffers, caps and events",
-    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
