@@ -21,13 +21,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
-/* FIXME: hack alert */
-#ifdef _MSC_VER
-#define DISABLE_FAULT_HANDLER
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
 #endif
 
-#include "tools.h"
+/* FIXME: hack alert */
+#ifdef WIN32
+#define DISABLE_FAULT_HANDLER
+#endif
 
 #include <string.h>
 #include <stdlib.h>
@@ -39,6 +40,9 @@
 #include <sys/wait.h>
 #endif
 #include <locale.h>             /* for LC_ALL */
+#include "gst/gst-i18n-app.h"
+
+#include <gst/gst.h>
 
 /* FIXME: This is just a temporary hack.  We should have a better
  * check for siginfo handling. */
@@ -52,73 +56,14 @@ extern volatile gboolean glib_on_error_halt;
 static void fault_restore (void);
 static void fault_spin (void);
 static void sigint_restore (void);
+static gboolean caught_intr = FALSE;
 #endif
 
-static gint max_iterations = 0;
-static guint64 iterations = 0;
-static guint64 sum = 0;
-static guint64 min = G_MAXINT64;
-static guint64 max = 0;
-static GstClock *s_clock;
 static GstElement *pipeline;
-static gboolean caught_intr = FALSE;
 static gboolean caught_error = FALSE;
-static gboolean need_new_state = FALSE;
-static GstElementState new_state;
+static gboolean tags = FALSE;
+static gboolean messages = FALSE;
 
-gboolean
-idle_func (gpointer data)
-{
-  gboolean busy;
-  GTimeVal tfthen, tfnow;
-  GstClockTimeDiff diff;
-
-  g_get_current_time (&tfthen);
-  busy = gst_bin_iterate (GST_BIN (data));
-  iterations++;
-  g_get_current_time (&tfnow);
-
-  diff = GST_TIMEVAL_TO_TIME (tfnow) - GST_TIMEVAL_TO_TIME (tfthen);
-
-  sum += diff;
-  min = MIN (min, diff);
-  max = MAX (max, diff);
-
-  if (need_new_state) {
-    gst_element_set_state (pipeline, new_state);
-    need_new_state = FALSE;
-  }
-
-  if (!busy || caught_intr || caught_error ||
-      (max_iterations > 0 && iterations >= max_iterations)) {
-    char *s_iterations;
-    char *s_sum;
-    char *s_ave;
-    char *s_min;
-    char *s_max;
-
-    gst_main_quit ();
-
-    /* We write these all to strings first because 
-     * G_GUINT64_FORMAT and gettext mix very poorly */
-    s_iterations = g_strdup_printf ("%" G_GUINT64_FORMAT, iterations);
-    s_sum = g_strdup_printf ("%" G_GUINT64_FORMAT, sum);
-    s_ave = g_strdup_printf ("%" G_GUINT64_FORMAT, sum / iterations);
-    s_min = g_strdup_printf ("%" G_GUINT64_FORMAT, min);
-    s_max = g_strdup_printf ("%" G_GUINT64_FORMAT, max);
-
-    g_print (_("Execution ended after %s iterations (sum %s ns, "
-            "average %s ns, min %s ns, max %s ns).\n"),
-        s_iterations, s_sum, s_ave, s_min, s_max);
-    g_free (s_iterations);
-    g_free (s_sum);
-    g_free (s_ave);
-    g_free (s_min);
-    g_free (s_max);
-  }
-
-  return busy;
-}
 
 #ifndef GST_DISABLE_LOADSAVE
 static GstElement *
@@ -139,6 +84,7 @@ xmllaunch_parse_cmdline (const gchar ** argv)
   }
 
   xml = gst_xml_new ();
+  /* FIXME guchar from gstxml.c */
   err = gst_xml_parse_file (xml, (guchar *) arg, NULL);
 
   if (err != TRUE) {
@@ -215,7 +161,7 @@ fault_handler_sighandler (int signum)
   fault_spin ();
 }
 
-#else
+#else /* USE_SIGINFO */
 
 static void
 fault_handler_sigaction (int signum, siginfo_t * si, void *misc)
@@ -240,7 +186,7 @@ fault_handler_sigaction (int signum, siginfo_t * si, void *misc)
 
   fault_spin ();
 }
-#endif
+#endif /* USE_SIGINFO */
 
 static void
 fault_spin (void)
@@ -287,7 +233,7 @@ fault_setup (void)
   sigaction (SIGSEGV, &action, NULL);
   sigaction (SIGQUIT, &action, NULL);
 }
-#endif
+#endif /* DISABLE_FAULT_HANDLER */
 
 static void
 print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
@@ -317,31 +263,45 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
   }
 }
 
-static void
-found_tag (GObject * pipeline, GstElement * source, GstTagList * tags)
-{
-  g_print (_("FOUND TAG      : found by element \"%s\".\n"),
-      GST_STR_NULL (GST_ELEMENT_NAME (source)));
-  gst_tag_list_foreach (tags, print_tag, NULL);
-}
-
-static void
-error_cb (GObject * object, GstObject * source, GError * error, gchar * debug)
-{
-  gst_element_default_error (object, source, error, debug);
-  caught_error = TRUE;
-}
-
 #ifndef DISABLE_FAULT_HANDLER
 /* we only use sighandler here because the registers are not important */
 static void
 sigint_handler_sighandler (int signum)
 {
+  g_print ("Caught interrupt -- ");
+
   sigint_restore ();
 
   caught_intr = TRUE;
 }
 
+static gboolean
+check_intr (GstElement * pipeline)
+{
+  if (!caught_intr) {
+    return TRUE;
+  } else {
+    GstBus *bus;
+    GstMessage *message;
+
+    caught_intr = FALSE;
+    g_print ("Pausing pipeline.\n");
+
+    bus = gst_element_get_bus (GST_ELEMENT (pipeline));
+    message = gst_message_new_warning (GST_OBJECT (pipeline),
+        NULL, "pipeline interrupted");
+    gst_bus_post (bus, message);
+
+    /* pipeline will wait for element to go to PAUSED */
+    gst_element_set_state (pipeline, GST_STATE_PAUSED);
+    g_print ("Pipeline paused.\n");
+
+    gst_object_unref (bus);
+
+    return FALSE;
+  }
+}
+
 static void
 sigint_setup (void)
 {
@@ -369,12 +329,12 @@ play_handler (int signum)
 {
   switch (signum) {
     case SIGUSR1:
-      new_state = GST_STATE_PLAYING;
-      need_new_state = TRUE;
+      g_print ("Caught SIGUSR1 - Play request.\n");
+      gst_element_set_state (pipeline, GST_STATE_PLAYING);
       break;
     case SIGUSR2:
-      new_state = GST_STATE_NULL;
-      need_new_state = TRUE;
+      g_print ("Caught SIGUSR2 - Stop request.\n");
+      gst_element_set_state (pipeline, GST_STATE_NULL);
       break;
   }
 }
@@ -389,8 +349,141 @@ play_signal_setup (void)
   sigaction (SIGUSR1, &action, NULL);
   sigaction (SIGUSR2, &action, NULL);
 }
+#endif /* DISABLE_FAULT_HANDLER */
+
+static gboolean
+event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
+{
+  GstBus *bus;
+  GstMessage *message = NULL;
+
+  bus = gst_element_get_bus (GST_ELEMENT (pipeline));
+
+#ifndef DISABLE_FAULT_HANDLER
+  g_timeout_add (50, (GSourceFunc) check_intr, pipeline);
 #endif
 
+  while (TRUE) {
+    message = gst_bus_poll (bus, GST_MESSAGE_ANY, blocking ? -1 : 0);
+
+    /* if the poll timed out, only when !blocking */
+    if (message == NULL) {
+      gst_object_unref (bus);
+      return FALSE;
+    }
+
+    if (messages) {
+      const GstStructure *s;
+
+      s = gst_message_get_structure (message);
+
+      g_print (_("Got Message from element \"%s\" (%s): "),
+          GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))),
+          gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
+      if (s) {
+        gchar *sstr;
+
+        sstr = gst_structure_to_string (s);
+        g_print ("%s\n", sstr);
+        g_free (sstr);
+      } else {
+        g_print ("no message details\n");
+      }
+    }
+
+    switch (GST_MESSAGE_TYPE (message)) {
+      case GST_MESSAGE_NEW_CLOCK:
+      {
+        GstClock *clock;
+
+        gst_message_parse_new_clock (message, &clock);
+
+        g_print ("New clock: %s\n", GST_OBJECT_NAME (clock));
+        gst_message_unref (message);
+        break;
+      }
+      case GST_MESSAGE_EOS:
+        g_print (_
+            ("Got EOS from element \"%s\".\n"),
+            GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))));
+        gst_message_unref (message);
+        gst_object_unref (bus);
+        return FALSE;
+      case GST_MESSAGE_TAG:
+        if (tags) {
+          GstTagList *tags;
+
+          gst_message_parse_tag (message, &tags);
+          g_print (_("FOUND TAG      : found by element \"%s\".\n"),
+              GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))));
+          gst_tag_list_foreach (tags, print_tag, NULL);
+          gst_tag_list_free (tags);
+        }
+        gst_message_unref (message);
+        break;
+      case GST_MESSAGE_WARNING:{
+        GError *gerror;
+        gchar *debug;
+
+        gst_message_parse_warning (message, &gerror, &debug);
+        if (debug) {
+          g_print ("WARNING: Element \"%s\" warns: %s\n",
+              GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))),
+              debug);
+        }
+        gst_message_unref (message);
+        if (gerror)
+          g_error_free (gerror);
+        g_free (debug);
+        break;
+      }
+      case GST_MESSAGE_ERROR:{
+        GError *gerror;
+        gchar *debug;
+
+        gst_message_parse_error (message, &gerror, &debug);
+        gst_object_default_error (GST_MESSAGE_SRC (message), gerror, debug);
+        gst_message_unref (message);
+        if (gerror)
+          g_error_free (gerror);
+        g_free (debug);
+        gst_object_unref (bus);
+        return TRUE;
+      }
+      case GST_MESSAGE_STATE_CHANGED:{
+        GstState old, new, pending;
+
+        gst_message_parse_state_changed (message, &old, &new, &pending);
+        if (GST_MESSAGE_SRC (message) == GST_OBJECT (pipeline) &&
+            target_state != GST_STATE_VOID_PENDING && new == target_state) {
+          gst_message_unref (message);
+          gst_object_unref (bus);
+          return FALSE;
+        }
+        if (!(old == GST_STATE_PLAYING && new == GST_STATE_PAUSED &&
+                GST_MESSAGE_SRC (message) == GST_OBJECT (pipeline))) {
+          gst_message_unref (message);
+          break;
+        }
+        g_print (_
+            ("Element \"%s\" has gone from PLAYING to PAUSED, quitting.\n"),
+            GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))));
+        /* cut out of the event loop if check_intr set us to PAUSED */
+        gst_message_unref (message);
+        gst_object_unref (bus);
+        return FALSE;
+      }
+      default:
+        /* just be quiet by default */
+        gst_message_unref (message);
+        break;
+    }
+  }
+
+  g_assert_not_reached ();
+  return TRUE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -398,39 +491,38 @@ main (int argc, char *argv[])
 
   /* options */
   gboolean verbose = FALSE;
-  gboolean tags = FALSE;
   gboolean no_fault = FALSE;
   gboolean trace = FALSE;
   gchar *savefile = NULL;
   gchar *exclude_args = NULL;
-  struct poptOption options[] = {
-    GST_TOOLS_POPT_VERSION,
-    {"tags", 't', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &tags, 0,
+  GOptionEntry options[] = {
+    {"tags", 't', 0, G_OPTION_ARG_NONE, &tags,
         N_("Output tags (also known as metadata)"), NULL},
-    {"verbose", 'v', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &verbose, 0,
+    {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
         N_("Output status information and property notifications"), NULL},
-    {"exclude", 'X', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, &exclude_args, 0,
+    {"messages", 'm', 0, G_OPTION_ARG_NONE, &messages,
+        N_("Output messages"), NULL},
+    {"exclude", 'X', 0, G_OPTION_ARG_NONE, &exclude_args,
         N_("Do not output status information of TYPE"), N_("TYPE1,TYPE2,...")},
 #ifndef GST_DISABLE_LOADSAVE
-    {"output", 'o', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, &savefile, 0,
+    {"output", 'o', 0, G_OPTION_ARG_STRING, &savefile,
         N_("Save xml representation of pipeline to FILE and exit"), N_("FILE")},
 #endif
-    {"no-fault", 'f', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &no_fault, 0,
+    {"no-fault", 'f', 0, G_OPTION_ARG_NONE, &no_fault,
         N_("Do not install a fault handler"), NULL},
-    {"trace", 'T', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &trace, 0,
+    {"trace", 'T', 0, G_OPTION_ARG_NONE, &trace,
         N_("Print alloc trace (if enabled at compile time)"), NULL},
-    {"iterations", 'i', POPT_ARG_INT | POPT_ARGFLAG_STRIP, &max_iterations, 0,
-        N_("Number of times to iterate pipeline"), NULL},
-    POPT_TABLEEND
+    {NULL}
   };
-
+  GOptionContext *ctx;
+  GError *err = NULL;
   gchar **argvn;
   GError *error = NULL;
   gint res = 0;
 
   free (malloc (8));            /* -lefence */
 
-#ifdef GETTEXT_PACKAGE
+#ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
@@ -438,8 +530,14 @@ main (int argc, char *argv[])
 
   gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE);
 
-  gst_init_with_popt_table (&argc, &argv, options);
-  gst_tools_print_version ("gst-launch-0.8");
+  ctx = g_option_context_new ("gst-launch");
+  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+  g_option_context_add_group (ctx, gst_init_get_option_group ());
+  if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
+    g_print ("Error initializing: %s\n", GST_STR_NULL (err->message));
+    exit (1);
+  }
+  g_option_context_free (ctx);
 
   /* FIXpopt: strip short args, too. We do it ourselves for now */
   j = 1;
@@ -472,7 +570,7 @@ main (int argc, char *argv[])
     if (!gst_alloc_trace_available ()) {
       g_warning ("Trace not available (recompile with trace enabled).");
     }
-    gst_alloc_trace_print_all ();
+    gst_alloc_trace_print_live ();
   }
 
   /* make a null-terminated version of argv */
@@ -492,29 +590,25 @@ main (int argc, char *argv[])
   if (!pipeline) {
     if (error) {
       fprintf (stderr, _("ERROR: pipeline could not be constructed: %s.\n"),
-          error->message);
+          GST_STR_NULL (error->message));
       g_error_free (error);
     } else {
       fprintf (stderr, _("ERROR: pipeline could not be constructed.\n"));
     }
     return 1;
   } else if (error) {
-    fprintf (stderr, _("WARNING: erroneous pipeline: %s\n"), error->message);
-    fprintf (stderr, _("         Trying to run anyway.\n"));
+    fprintf (stderr, _("WARNING: erroneous pipeline: %s\n"),
+        GST_STR_NULL (error->message));
     g_error_free (error);
+    return 1;
   }
 
   if (verbose) {
     gchar **exclude_list =
         exclude_args ? g_strsplit (exclude_args, ",", 0) : NULL;
     g_signal_connect (pipeline, "deep_notify",
-        G_CALLBACK (gst_element_default_deep_notify), exclude_list);
+        G_CALLBACK (gst_object_default_deep_notify), exclude_list);
   }
-  if (tags) {
-    g_signal_connect (pipeline, "found-tag", G_CALLBACK (found_tag), NULL);
-  }
-  g_signal_connect (pipeline, "error", G_CALLBACK (error_cb), NULL);
-
 #ifndef GST_DISABLE_LOADSAVE
   if (savefile) {
     gst_xml_write_file (GST_ELEMENT (pipeline), fopen (savefile, "w"));
@@ -522,6 +616,8 @@ main (int argc, char *argv[])
 #endif
 
   if (!savefile) {
+    GstState state, pending;
+    GstStateChangeReturn ret;
 
     if (!GST_IS_BIN (pipeline)) {
       GstElement *real_pipeline = gst_element_factory_make ("pipeline", NULL);
@@ -533,41 +629,77 @@ main (int argc, char *argv[])
       gst_bin_add (GST_BIN (real_pipeline), pipeline);
       pipeline = real_pipeline;
     }
+    fprintf (stderr, _("Setting pipeline to PAUSED ...\n"));
+    ret = gst_element_set_state (pipeline, GST_STATE_PAUSED);
 
-    fprintf (stderr, _("RUNNING pipeline ...\n"));
-    if (gst_element_set_state (pipeline,
-            GST_STATE_PLAYING) == GST_STATE_FAILURE) {
-      fprintf (stderr, _("ERROR: pipeline doesn't want to play.\n"));
-      res = -1;
-      goto end;
+    switch (ret) {
+      case GST_STATE_CHANGE_FAILURE:
+        fprintf (stderr, _("ERROR: Pipeline doesn't want to pause.\n"));
+        res = -1;
+        event_loop (pipeline, FALSE, GST_STATE_VOID_PENDING);
+        goto end;
+      case GST_STATE_CHANGE_NO_PREROLL:
+        fprintf (stderr, _("ERROR: Pipeline can't PREROLL ...\n"));
+        break;
+      case GST_STATE_CHANGE_ASYNC:
+        fprintf (stderr, _("Pipeline is PREROLLING ...\n"));
+        caught_error = event_loop (pipeline, TRUE, GST_STATE_PAUSED);
+        if (caught_error) {
+          fprintf (stderr, _("ERROR: pipeline doesn't want to preroll.\n"));
+          goto end;
+        }
+        state = GST_STATE_PAUSED;
+        /* fallthrough */
+      case GST_STATE_CHANGE_SUCCESS:
+        fprintf (stderr, _("Pipeline is PREROLLED ...\n"));
+        break;
     }
 
-    s_clock = gst_bin_get_clock (GST_BIN (pipeline));
+    caught_error = event_loop (pipeline, FALSE, GST_STATE_VOID_PENDING);
 
-    if (!GST_FLAG_IS_SET (GST_OBJECT (pipeline), GST_BIN_SELF_SCHEDULABLE)) {
-      g_idle_add (idle_func, pipeline);
-      gst_main ();
+    if (caught_error) {
+      fprintf (stderr, _("ERROR: pipeline doesn't want to preroll.\n"));
     } else {
-      g_print ("Waiting for the state change... ");
-      gst_element_wait_state_change (pipeline);
-      g_print ("got the state change.\n");
-    }
-    if (caught_intr) {
-      g_print ("Caught interrupt.\n");
-      res = 2;
-    }
-    if (caught_error)
-      res = 3;
+      GTimeVal tfthen, tfnow;
+      GstClockTimeDiff diff;
 
+      fprintf (stderr, _("Setting pipeline to PLAYING ...\n"));
+      if (gst_element_set_state (pipeline,
+              GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE) {
+        fprintf (stderr, _("ERROR: pipeline doesn't want to play.\n"));
+        res = -1;
+        goto end;
+      }
+
+      g_get_current_time (&tfthen);
+      caught_error = event_loop (pipeline, TRUE, GST_STATE_VOID_PENDING);
+      g_get_current_time (&tfnow);
+
+      diff = GST_TIMEVAL_TO_TIME (tfnow) - GST_TIMEVAL_TO_TIME (tfthen);
+
+      g_print (_("Execution ended after %" G_GUINT64_FORMAT " ns.\n"), diff);
+    }
+    while (g_main_context_iteration (NULL, FALSE));
+
+    fprintf (stderr, _("Setting pipeline to PAUSED ...\n"));
+    gst_element_set_state (pipeline, GST_STATE_PAUSED);
+    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
+    fprintf (stderr, _("Setting pipeline to READY ...\n"));
+    gst_element_set_state (pipeline, GST_STATE_READY);
+    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
+
+  end:
+    fprintf (stderr, _("Setting pipeline to NULL ...\n"));
     gst_element_set_state (pipeline, GST_STATE_NULL);
+    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
   }
 
-end:
-
-  gst_object_unref (GST_OBJECT (pipeline));
+  fprintf (stderr, _("FREEING pipeline ...\n"));
+  gst_object_unref (pipeline);
 
+  gst_deinit ();
   if (trace)
-    gst_alloc_trace_print_all ();
+    gst_alloc_trace_print_live ();
 
   return res;
 }
